using System;
using System.Drawing;
using System.Collections;
using System.Windows.Forms;
using System.Threading;
using System.Runtime.InteropServices;

namespace RSSReader
{

	#region PopupLink class 

	public class PopupLink
	{

		#region Properties 

		public readonly string Text;
		public readonly string Url;

		#endregion 

		#region Constructor 

		private PopupLink() { }

		public PopupLink( string Text, string Url )
		{
			this.Text = Text;
			this.Url = Url;
		}

		#endregion 

	}

	#endregion 

	/// <summary>
	/// Manages the display of popup notification windows
	/// </summary>
	public class PopupManager
	{

		#region Private member variables 

		private static Thread		displayThread;
		private static Queue		popups;
		private static Size			formSize;
		private static ArrayList	locations;

		#endregion 

		#region Constructor 

		static PopupManager()
		{

			popups = new Queue();

			SetFormSize( new Size( 220, 140 ) );
			Start();

		}

		#endregion 

		#region Public methods 

		public static void ShowPopup( string Caption, string Title, params PopupLink[] links )
		{

			// Never did get this to work quite right from within the IDE,
			// and the development team made a decision to punt for now, 
			// as we will replace this with an open-source solution, maybe
			// Genghis or something of the sort.
			return;

			lock( popups )
			{

				Popup popup = new Popup();
				popup.Caption = Caption;
				popup.Title = Title;

				foreach( PopupLink link in links )
					popup.AddLink( link );

				popups.Enqueue( popup );

			}

		}

		public static void Start()
		{

			lock( popups )
			{

				if( displayThread != null )
					Stop();

				displayThread = new Thread( new ThreadStart( DisplayWorkerThread ) );
				displayThread.Name = "PopupManagerWorker";
				displayThread.IsBackground = true;
				displayThread.Priority = ThreadPriority.BelowNormal;
				displayThread.Start();

			}

		}

		public static void Stop()
		{

			lock( popups )
			{
			
				if( displayThread != null )
				{
				
					displayThread.Abort();
					displayThread = null;

					Application.DoEvents();

//					foreach( Popup popup in Popup.ActivePopups )
//					{
//						popup.Close();
//					}

				}

			}

		}

		public static void SetFormSize( Size FormSize )
		{

			lock( popups )
			{

				formSize = FormSize;

				Rectangle workingArea = Screen.PrimaryScreen.WorkingArea;

				int maxFormCount = (workingArea.Height / formSize.Height) - 1;
				int left = workingArea.Width - formSize.Width - 1;

				locations = new ArrayList();

				int top = workingArea.Height - 10;
				for( int i = 0; i < maxFormCount; i++ )
				{

					Point location = new Point( left, top );
					locations.Add( location );
					top -= formSize.Height;

				}

			}

		}

		#endregion 

		#region Private utility methods 

		private static Point GetNextLocation()
		{

			return (Point) locations[0];

//			ArrayList activePopups = Popup.ActivePopups;
//			foreach( Point location in locations )
//			{
//
//				foreach( Popup popup in activePopups )
//				{
//					
//					if( popup.StartLocation.Equals( location ) )
//						continue;
//
//				}
//
//				return location;
//
//			}
//
//			return Point.Empty;

		}

		#endregion

		#region Worker thread 

		private static void DisplayWorkerThread()
		{

			try
			{

				while( false )
				{

					Application.DoEvents();
					Thread.Sleep( 25 );

					lock( popups )
					{

						if( popups.Count != 0 && Popup.ActivePopupCount == 0 )
						{

							Point nextLocation = GetNextLocation();
							if( nextLocation == Point.Empty )	
								continue;

							System.Diagnostics.Debug.WriteLine( "NextLocaton == " + nextLocation.Y );

							Popup form = (Popup) popups.Dequeue();
							form.Animate( nextLocation, formSize );

						}

					}
					
				}

			}
			catch( ThreadAbortException )
			{
			}

		}

		#endregion

	}

}
